from mykit.kit.utils import printer

from ..constants import __version__, SOFTWARE_DIST_NAME, SOFTWARE_REPO


def _writer(author, email, gh_repo, main_branch_name):
    return f"""

# This file was generated by {SOFTWARE_DIST_NAME}-v{__version__}.
# Please make sure to be careful when modifying the values below. For more information, visit: {SOFTWARE_REPO}

name: Rebuild docs

on:
  push:
    branches:
      - {main_branch_name}

    paths:
      - 'docs/**'  # only rebuild if 'docs/' folder is modified

  # Allows you to run this workflow manually from the Actions tab
  workflow_dispatch:

permissions:
  contents: write

jobs:
  deploy:
    runs-on: ubuntu-latest

    steps:

      - name: Checkout {main_branch_name} branch
        uses: actions/checkout@v3
        with:
          ref: {main_branch_name}

      - name: Creating NICS working directory
        run: |
          cd ..
          mkdir __nics_work_dir__
          cd __nics_work_dir__
          cp -r ../{gh_repo}/docs/ .

      - name: Set up Python
        uses: actions/setup-python@v2
        with:
          python-version: 3.8

      - name: Installing NICS
        run: |
          python -m pip install --upgrade pip
          pip install nics=={__version__}

      - name: Checkout docs branch
        uses: actions/checkout@v3
        with:
          ref: docs

      - name: Compile
        run: |
          cd ..
          cd __nics_work_dir__
          nics _compile "$(pwd)/docs" "$(pwd)/../{gh_repo}"

      - name: Deploy
        run: |
          git config user.name "{author} (via NICS)"
          git config user.email "{email}"
          git add .
          git commit -m "NICS rebuilds the docs"
          git push
"""


def workflows_writer(pth, author, email, gh_repo, main_branch_name):
    printer(f'INFO: Writing GitHub workflow file.')

    text = _writer(author, email, gh_repo, main_branch_name)
    with open(pth, 'w') as f:
        f.write(text)

    printer(f'INFO: Done, {repr(pth)} is created.')


import os
import sys

from mykit.kit.utils import printer


def inspect(container_dir_pth, workflow_file_pth):
    
    ## the container ('docs/' folder) should not exist
    if os.path.isdir(container_dir_pth):
        printer(f"ERROR: Folder {repr(container_dir_pth)} already exists.")
        sys.exit(1)

    ## the GitHub Action workflow file 'rebuild-docs.yml' should not exist
    if os.path.isfile(workflow_file_pth):
        printer(f"ERROR: File {repr(workflow_file_pth)} already exists.")
        sys.exit(1)


import os
import pkg_resources


try:
    __version__ = pkg_resources.get_distribution('nics').version
except pkg_resources.DistributionNotFound:  # this exception occurred during development (before the software installed via pip)
    __version__ = 'dev'


SOFTWARE_REPO = 'https://github.com/nvfp/now-i-can-sleep'
SOFTWARE_NAME = 'Now I Can Sleep'
SOFTWARE_DIST_NAME = 'nics'  # distribution name

ROOT_DIR_PTH = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
DIST_DIR_PTH = os.path.join(ROOT_DIR_PTH, SOFTWARE_DIST_NAME)
MAIN_DIR_PTH = os.path.join(ROOT_DIR_PTH, SOFTWARE_DIST_NAME, 'main')

TEMPLATE_DIR_PTH = os.path.join(MAIN_DIR_PTH, '_template')
TMPL_DOCS_DIR_PTH = os.path.join(TEMPLATE_DIR_PTH, 'docs')
TMPL_WEB_DIR_PTH = os.path.join(TEMPLATE_DIR_PTH, 'web')

SETTINGS_KEYS = [
    'author',
    'color_hue',
    'show_credit',
    '_gh_username',
    '_gh_repo',
]


import os
import re


def _inspect_tree_recursively(pth):

    for i in os.listdir(pth):        
        pth2 = os.path.join(pth, i)

        if os.path.isdir(pth2):
            ## check dir name format
            if re.match(r'(?:\d+ -- )?[\w -.]+ -- [\w -]+', i) is None:
                raise AssertionError(f'Dir name {repr(i)} format is invalid.')
            ## inspect the subdir
            _inspect_tree_recursively(pth2)
        else:
            ## non-markdown files don't need to be checked (only check .md files)
            if i.endswith('.md'):
                ## check file name format
                if i != 'index.md':
                    if re.match(r'(?:\d+ -- )?[\w -.]+ -- [\w -]+\.md$', i) is None:
                        raise AssertionError(f'File name {repr(i)} format is invalid.')


def inspect_the_container(container):

    ## 'tree/' folder
    if not os.path.isdir( os.path.join(container, 'tree') ):
        raise AssertionError("Couldn't find 'tree/' in the container.")

    ## homepage 'tree/index.md' file
    if not os.path.isfile( os.path.join(container, 'tree', 'index.md') ):
        raise AssertionError("Couldn't find 'tree/index.md' in the container.")

    ## settings file
    if not os.path.isfile( os.path.join(container, 'settings.txt') ):
        raise AssertionError("Couldn't find 'settings.txt' in the container.")

    ## inspecting the 'tree/' folder
    _inspect_tree_recursively( os.path.join(container, 'tree') )


from mykit.kit.utils import printer


def update_sass_constants(D_SASS_CONSTANTS, color_hue):

    text = f'$hue-signature: {color_hue};'
    with open(D_SASS_CONSTANTS, 'w') as f: f.write(text)
    printer(f'INFO: Updated sass constants {repr(D_SASS_CONSTANTS)}.')


import os
import shutil

from mykit.kit.utils import printer, print_screen

from ..constants import TMPL_DOCS_DIR_PTH
from .inspect import inspect
from .workflows_writer import workflows_writer
from .settings_writer import settings_writer


def run():

    CWD = os.getcwd()

    WORKFLOW_FILE_PTH = os.path.join(CWD, '.github', 'workflows', 'rebuild-docs.yml')
    CONTAINER_DIR_PTH = os.path.join(CWD, 'docs')
    SETTINGS_FILE_PTH = os.path.join(CWD, 'docs', 'settings.txt')

    ## inspection
    inspect(CONTAINER_DIR_PTH, WORKFLOW_FILE_PTH)

    ## intro
    print_screen(
        'Welcome to NICS!\n'
        '================\n\n'
    )

    ## saving user's information
    author = input('Enter your name: ')
    email = input('Enter your email: ')
    gh_username = input('Enter your GitHub username: ')
    gh_repo = input('Enter this GitHub repository name: ')
    main_branch_name = input("Enter the main branch name (e.g., master): ")

    ## handle the case where the '.github/workflows/' directory does not exist yet
    def handle_workflow_dirs():
        pth = os.path.join(CWD, '.github')
        if not os.path.isdir(pth):
            os.mkdir(pth)
            printer(f"INFO: Folder {repr(pth)} is created.")
        pth = os.path.join(CWD, '.github', 'workflows')
        if not os.path.isdir(pth):
            os.mkdir(pth)
            printer(f"INFO: Folder {repr(pth)} is created.")
    handle_workflow_dirs()

    workflows_writer(WORKFLOW_FILE_PTH, author, email, gh_repo, main_branch_name)
    
    printer(f"INFO: Copying 'docs/' folder.")
    shutil.copytree(TMPL_DOCS_DIR_PTH, CONTAINER_DIR_PTH)
    printer(f'INFO: Done, {repr(CONTAINER_DIR_PTH)} is created.')

    settings_writer(SETTINGS_FILE_PTH, author, gh_username, gh_repo)

    ## outro
    print_screen(f"""
Almost done, now you need to do these final steps:
1. Create docs branch
   - git commit -am "NICS init"
   - git checkout --orphan docs
   - git rm -rf .
   - git commit --allow-empty -m init
   - git push origin docs
2. Activate the GitHub Pages
   - Visit https://github.com/{gh_username}/{gh_repo}/settings/pages
   - Under 'Build and deployment' section,
     - For 'Source', select 'Deploy from a branch'
     - For 'Branch', select 'docs' branch
     - Click the 'Save' button
3. Back to {main_branch_name} branch
   - git checkout {main_branch_name}
   - git push

That's it! The documentation will be at https://{gh_username}.github.io/{gh_repo} .
Visit https://nvfp.github.io/now-i-can-sleep/usage/init-command to visually see the final steps above."""
    )